import subprocess
from typing import List

from pydmt.configs import ConfigSubprocess


def check_call(args: List[str]):
    if ConfigSubprocess.print_command:
        print(f"running {args}")
    if ConfigSubprocess.quiet:
        subprocess.check_call(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    else:
        subprocess.check_call(args)



import subprocess

git_last_tag = subprocess.check_output(
    ["git", "describe", "--abbrev=0", "--tags"], stderr=subprocess.DEVNULL
).decode().rstrip()
git_describe = subprocess.check_output(
    ["git", "describe"], stderr=subprocess.DEVNULL
).decode().rstrip()
git_version = ".".join(git_describe.split("-"))



import os
import os.path
import importlib
import glob
import pprint
from typing import Callable, List, Dict


def make_console_script(package_name: str, main: Callable):
    """
    return a string suitable to be used as the section 'console_scripts'
    of the 'entry_points' parameters of setup.py.

    example: 'pycmdtools=pycmdtools.endpoints.main:main'
    :param package_name:
    :param main:
    :return:
    """
    return f"{package_name}={getattr(main, '__module__')}:{main.__name__,}"


def array_indented(level: int, array: List[str], quote_char='\'', comma_after=False) -> str:
    """
    return an array indented according to indent level
    :param level:
    :param array:
    :param quote_char:
    :param comma_after:
    :return:
    """
    out = "[\n"
    for x in array:
        out += (((level + 1) * 4) * " ") + '{qc}{x}{qc}'.format(qc=quote_char, x=x) + ",\n"
    out += ((level * 4) * " ") + "]"
    if comma_after:
        out += ","
    return out


def dict_indented(level: int, dictionary: Dict[str, List[int]], quote_char='\'', comma_after=False) -> str:
    """
    return an dict indented according to indent level
    :param level:
    :param dictionary:
    :param quote_char:
    :param comma_after:
    :return:
    """
    out = "{\n"
    for k, v in dictionary.items():
        spaces = (((level + 1) * 4) * " ")
        out += '{spaces}{qc}{k}{qc}: {v},\n'.format(
            spaces=spaces,
            qc=quote_char,
            k=k,
            v=v,
        )
    spaces = ((level * 4) * " ")
    out += f"{spaces}}}"
    if comma_after:
        out += ","
    return out


def find_packages(path: str) -> List[str]:
    """
    A better version of find_packages than what setuptools offers
    This function needs to be deterministic.
    :param path:
    :return:
    """
    ret = []
    for root, _dir, files in os.walk(path):
        if '__init__.py' in files:
            ret.append(root.replace("/", "."))
    return sorted(ret)


def get_list_unquoted(a_list: List[str]) -> str:
    """
    The exact format of this output is to be used in python code from templates,
    that is why the left bracket ([) does not have space following it and the
    same for the right bracker (]).
    """
    s = "["
    s += ", ".join(a_list)
    s += "]"
    return s


def get_list_quoted(a_list: List[str]) -> str:
    """
    The exact format of this output is to be used in python code from templates,
    that is why the left bracket ([) does not have space following it and the
    same for the right bracker (]).
    """
    quoted = map(lambda x: f"\"{x}\"", a_list)
    s = "["
    s += ", ".join(quoted)
    s += "]"
    return s


def get_package_name():
    mod = importlib.import_module("config.python")
    if hasattr(mod, "package_name"):
        return getattr(mod, "package_name")
    return os.path.basename(os.getcwd())


def get_attr(attr: str):
    mod = importlib.import_module("config.python")
    if hasattr(mod, attr):
        return getattr(mod, attr)
    mod = importlib.import_module("default.python")
    if hasattr(mod, attr):
        return getattr(mod, attr)
    # TODO: need a better exception type (make my own)
    raise ValueError(f"cannot find {attr}")


def get_license_type():
    return get_attr("license_type")


def get_platforms():
    return get_attr("platforms")


def get_classifiers():
    return get_attr("classifiers")


def hlp_source_under(folder):
    """
    this function finds all the python packages under a folder and
    write the 'packages' and 'package_dir' entries for a python setup.py
    script
    """
    # walk the folder and find the __init__.py entries for packages.
    packages = []
    package_dir = {}
    for root, _dirs, files in os.walk(folder):
        for file in files:
            if file != '__init__.py':
                continue
            full = os.path.dirname(os.path.join(root, file))
            relative = os.path.relpath(full, folder)
            packages.append(relative)
            package_dir[relative] = full
    # we use pprint because we want the order to always remain the same
    return f"packages={sorted(packages)},\npackage_dir={pprint.pformat(package_dir)}"


def hlp_files_under(dest_folder, pat):
    return f"('{dest_folder}', {[x for x in glob.glob(pat) if os.path.isfile(x)]})"


def make_hlp_project_keywords(d):
    def hlp_project_keywords():
        return f"{d.project_keywords.split()}"
    return hlp_project_keywords


def make_hlp_project_platforms(d):
    def hlp_project_platforms():
        return f"{d.project_platforms.split()}"

    return hlp_project_platforms


def make_hlp_project_classifiers(d):
    def hlp_project_classifiers():
        cls_list = d.project_classifiers.split('\n')
        cls_list = [x.strip()[1:-1] for x in cls_list]
        return f"{cls_list}"

    return hlp_project_classifiers


def make_hlp_wrap(level):
    def hlp_wrap(t):
        return t.replace('\n', '\n' + '\t' * level)
    return hlp_wrap



import json
from urllib.request import urlopen
import yaml

from jsonschema import validate

from pydmt.utils.filesystem import mkdir_touch

from pydmt.api.one_source_one_target import OneSourceOneTarget

METADATA = "metadata"
SCHEMA_URL_JSON = "schema_url_json"
SCHEMA_URL_YAML = "schema_url_yaml"


class BuilderYaml(OneSourceOneTarget):
    def build(self):
        with open(self.source) as yaml_stream:
            datas = yaml.safe_load_all(yaml_stream)
            for data in datas:
                if isinstance(data, dict):
                    if METADATA in data:
                        metadata = data[METADATA]
                        if SCHEMA_URL_JSON in metadata:
                            schema_url = data[METADATA][SCHEMA_URL_JSON]
                            with urlopen(schema_url) as f:
                                schema = json.loads(f.read())
                                validate(data, schema)
                        if SCHEMA_URL_YAML in metadata:
                            schema_url = data[METADATA][SCHEMA_URL_YAML]
                            with urlopen(schema_url) as f:
                                schema = yaml.safe_load(f.read())
                                validate(data, schema)
            mkdir_touch(self.target)



import os

from pydmt.api.feature import Feature
from pydmt.builders.make import BuilderMake
from pydmt.core.pydmt import PyDMT


class FeatureMake(Feature):
    def setup(self, pydmt: PyDMT) -> None:
        if os.path.isfile("Makefile"):
            pydmt.add_builder(BuilderMake(
                source="Makefile",
                target="out/make.stamp",
            ))



import hashlib
from typing import List

from pydmt.utils.filesystem import files_under_folders


BLOCK_SIZE = 65536


def hex_digest_files(filenames: List[str], algorithm_name: str) -> str:
    hash_object = hashlib.new(algorithm_name)
    for filename in filenames:
        with open(filename, 'rb') as file_handle:
            buf = file_handle.read(BLOCK_SIZE)
            while len(buf) > 0:
                hash_object.update(buf)
                buf = file_handle.read(BLOCK_SIZE)
    return hash_object.hexdigest()


def hex_digest_file(filename: str, algorithm_name: str) -> str:
    return hex_digest_files(filenames=[filename], algorithm_name=algorithm_name)


def sha1_file(filename: str) -> str:
    return hex_digest_file(filename=filename, algorithm_name="sha1")


def sha1_files(filenames: List[str]) -> str:
    return hex_digest_files(filenames=filenames, algorithm_name="sha1")


def sha1_folders(folders: List[str]) -> str:
    return sha1_files(filenames=files_under_folders(folders=folders))


def sha1_files_folders(files: List[str], folders: List[str]) -> str:
    f = files
    f.extend(files_under_folders(folders=folders))
    return sha1_files(f)



import setuptools


def get_readme():
    with open('README.rst') as f:
        return f.read()


setuptools.setup(
    # the first three fields are a must according to the documentation
    name="pydmt",
    version="0.1.69",
    packages=[
        'pydmt',
        'pydmt.api',
        'pydmt.builders',
        'pydmt.core',
        'pydmt.features',
        'pydmt.helpers',
        'pydmt.utils',
    ],
    # from here all is optional
    description="python dependency management tool",
    long_description=get_readme(),
    long_description_content_type="text/x-rst",
    author="Mark Veltzer",
    author_email="mark.veltzer@gmail.com",
    maintainer="Mark Veltzer",
    maintainer_email="mark.veltzer@gmail.com",
    keywords=[
        'pydmt',
        'cons',
        'scons',
        'software construction',
        'make',
        'cmake',
        'maven',
        'mvn',
    ],
    url="https://veltzer.github.io/pydmt",
    download_url="https://github.com/veltzer/pydmt",
    license="MIT",
    platforms=[
        'python3',
    ],
    install_requires=[
        'pyfakeuse',
        'pylogconf',
        'pytconf',
        'mako',
        'Sphinx',
        'pyyaml',
        'jsonschema',
        'venv-run',
        'gitpython',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
    ],
    entry_points={"console_scripts": [
        'pydmt=pydmt.main:main',
    ]},
)



import os

from pydmt.api.feature import Feature
from pydmt.builders.sphinx import BuilderSphinx
from pydmt.core.pydmt import PyDMT

SPHINX_FOLDER = "sphinx"


class FeatureSphinx(Feature):
    def setup(self, pydmt: PyDMT) -> None:
        if not os.path.isdir(SPHINX_FOLDER):
            return
        pydmt.add_builder(BuilderSphinx())
